#include "vnlist.h"


struct vnlist_item *vl_add_item(struct vnlist_item **first, struct vnlist_item *item)
{
	struct vnlist_item *new_entry = (struct vnlist_item *)calloc(sizeof(struct vnlist_item), 1);
	if (new_entry == NULL) return NULL;
	*new_entry = *item;
	new_entry->next = *first;
	*first = new_entry;
}

struct vnlist_item *vl_find_name(struct vnlist_item *first, const char *name)
{
	struct vnlist_item *entry = first;
	while(entry) {
		if (strcmp(name, entry->name) == 0) break;
		entry = entry->next;
	}
	return entry;
}

struct vnlist_item *vl_find_id(struct vnlist_item *first, vnode_id id)
{
	struct vnlist_item *entry = first;
	while(entry) {
		if (id == entry->id) break;
		entry = entry->next;
	}
	return entry;
}

void vl_free(struct vnlist_item *first)
{
	struct vnlist_item *entry = first, *next;
	while(entry) {
		next = entry->next;
		free(entry);
		entry = next;
	}
}

struct dir_item *di_add_dir(struct dir_item **first, vnode_id id)
{
	struct dir_item *new_dirent = (struct dir_item *)calloc(sizeof(struct dir_item), 1);
	if (new_dirent == NULL) return NULL;
	new_dirent->id = id;
	new_dirent->next = *first;
	*first = new_dirent;
	return new_dirent;
}

struct dir_item *di_find_dir(struct dir_item *first, vnode_id id)
{
	struct dir_item *dirent = first;
	while(dirent) {
		if (id == dirent->id) break;
		dirent = dirent->next;
	}
	return dirent;
}

void di_free(struct dir_item *first)
{
	struct dir_item *dirent = first, *next;
	while(dirent) {
		next = dirent->next;
		vl_free(dirent->first);
		free(dirent);
		dirent = next;
	}
}

struct vnlist_item *vl_find_r(struct dir_item *first, vnode_id id, struct dir_item **dir)
{
	struct dir_item *dirent = first;
	while(dirent) {
		struct vnlist_item *entry = dirent->first;
		while(entry) {
			if (id == entry->id) {
				*dir = dirent;
				return entry;
			}
			entry = entry->next;
		}
		dirent = dirent->next;
	}
	return NULL;
}
